#region usings
using System;
using System.ComponentModel.Composition;

using VVVV.PluginInterfaces.V1;
using VVVV.PluginInterfaces.V2;
using VVVV.Utils.VColor;
using VVVV.Utils.VMath;
using System.Linq;

using SlimDX;
using VVVV.Core.Logging;
#endregion usings

namespace VVVV.Nodes
{	
	
	
	
	public abstract class InputMorph<T> : IPluginEvaluate
	{
		[Input("Switch")]
		ISpread<double> FMorph;
	
		[Input("Input", IsPinGroup = true)]
		ISpread<ISpread<T>> FInput;

		[Output("Output")]
		ISpread<T> FOutput;
		

		
		//called when data for any output pin is requested
		public void Evaluate(int SpreadMax)
		{
			SpreadMax = SpreadUtils.SpreadMax(FInput.ToArray());
			SpreadMax = SpreadMax.CombineWith(FMorph);

			FOutput.SliceCount = SpreadMax;
			
			for (int i = 0; i < SpreadMax; i++)
			{
				double morph = FMorph[i];
				
				int firstPinIndex = (int)Math.Floor(morph);
				int secondPinIndex = firstPinIndex+1;
				
				var firstSpread = FInput[firstPinIndex];
				var secondSpread = FInput[secondPinIndex];
				
				FOutput[i] = Morph(
					firstSpread[i],
					secondSpread[i], 
					morph % 1);
			}
		}
		
		protected abstract T Morph(T a, T b, double morph);
	}

	public static class MyStringHelper
	{
		public static string Morph(string a, string b, double morph)
		{
			
			var la = a.Length;
			var lb = b.Length;
			var wordLength = (int)Math.Round(VMath.Lerp(la, lb, morph));
			
			var chars = new char[wordLength];
			
			for(int i=0; i<wordLength; i++)
			{
				var ca = a[i % la];
				var cb = b[i % lb];
				
				var ia = (int)ca;
				var ib = (int)cb;
				
				var iout = (int)Math.Round(VMath.Lerp(ca, cb, morph));
				var cout = (char)iout;
				
				chars[i] = cout;
			}
			
			return new string(chars);
		}
		
		public static ISpread<string> Morph(ISpread<string> a, ISpread<string> b, double morph)
		{
			var linesA = a.SliceCount;
			var linesB = b.SliceCount;
			
			var linesResult = (int)Math.Round(VMath.Lerp(linesA, linesB, morph));
			var result = new Spread<string>();
		
			for (var i=0; i < linesResult; i++)
			{
				var lineA = a[i];
				var lineB = b[i];
				
				var myString = MyStringHelper.Morph(lineA, lineB, morph);
				
				result.Add(myString);
			}
			
			return result;
		}
	}
		
	[PluginInfo(Name = "InputMorph", Category = "String")]
	public class InputMorphString : InputMorph<string>
	{
		protected override string Morph(string a, string b, double morph)
		{
			return MyStringHelper.Morph(a, b, morph);			
		}
	}
	
	[PluginInfo(Name = "InputMorph", Category = "Tranform")]
	public class InputMorphTransform : InputMorph<Matrix>
	{
		protected override Matrix Morph(Matrix a, Matrix b, double morph)
		{			
			return Matrix.Lerp(a, b, (float)morph);
		}
	}
	
	//write a morph for RGBAColor ----> InputMorph<RGBAColor>
	
	[PluginInfo(Name = "InputMorph", Category = "Color")]
	public class InputMorphColor : InputMorph<RGBAColor>
	{
		protected override RGBAColor Morph(RGBAColor a, RGBAColor b, double morph)
		{
			return VColor.LerpRGBA(a, b, morph);
		}
	}
	
	// write a morph for ISpread<string>  ----> ...
	
	[PluginInfo(Name = "InputMorph", Category = "Strings")]
	public class InputMorphStrings : InputMorph<ISpread<string>>
	{
		protected override ISpread<string> Morph(ISpread<string> a, ISpread<string> b, double morph)
		{
			return MyStringHelper.Morph(a, b, morph);
		}
	}
	// write a Exhibit morph
	
	[PluginInfo(Name = "InputMorph", Category = "Exhibit")]
	public class InputMorphExhibit : InputMorph<Exhibit>
	{
		protected override Exhibit Morph(Exhibit a, Exhibit b, double morph)
		{
			return new Exhibit()
			{
				Color = VColor.LerpRGBA(a.Color, b.Color, morph),
				Transformation = Matrix.Lerp(a.Transformation, b.Transformation, (float)morph),
				Words = MyStringHelper.Morph(a.Words, b.Words, morph)
			};
		}
	}
}
